@@ -1,3 +1,4 @@
+/* -*- mode:C; indent-tabs-mode:nill; tab-width:8; c-basic-offset:8 -*- */
 /*
  * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
@@ -21,32 +22,24 @@
 #include "blockops.h"
 #include "blocks.h"
 #include "field.h"
-#include "gnome-canvas-pimage.h"
 
-GnomeCanvasItem *BlockOps::generateItem(int x, int y, int colour)
-{
-	GnomeCanvasItem *it = gnome_canvas_item_new(
-  			gnome_canvas_root(GNOME_CANVAS(fieldDisplay->getWidget())),
-  			gnome_canvas_pimage_get_type(),
-  			"image", pic[colour],
-  			"x", (double) x * BLOCK_SIZE,
-  			"y", (double) y * BLOCK_SIZE,
-			"width", (double) BLOCK_SIZE,
-			"height", (double) BLOCK_SIZE,
-  			NULL);
+#define NCOLOURS 7
 
-	return it;
-}
 
-BlockOps::BlockOps(Field *f)
+BlockOps::BlockOps() :
+     blocknr (0),
+     rot (0),
+     color (0),
+     useTarget (false)
 {
 	field = new Block*[COLUMNS];
-	fieldDisplay = f;
 	
+	posx = COLUMNS / 2;
+	posy = 0;
+
 	for (int i = 0; i < COLUMNS; ++i)
 		field[i] = new Block[LINES];
 
-	field_initialized = false;
 	emptyField();
 }
 
@@ -62,7 +55,7 @@ bool
 BlockOps::blockOkHere(int x, int y, int b, int r)
 {
 	x -= 2;
-	
+
 	for (int x1 = 0; x1 < 4; ++x1)
 	{
 		for (int y1 = 0; y1 < 4; ++y1)
@@ -77,6 +70,7 @@ BlockOps::blockOkHere(int x, int y, int b, int r)
 				return false;
 		}
 	}
+
 	return true;	
 }
 
@@ -102,6 +96,7 @@ BlockOps::getLinesToBottom()
 				lines = tmp;
 		}
 	}
+
 	return lines;
 }
 
@@ -182,6 +177,46 @@ BlockOps::moveBlockDown()
 	return fallen;
 }
 
+void 
+BlockOps::clearTarget ()
+{
+	for (int x = 0; x < COLUMNS; ++x)
+		for (int y = 0; y < LINES; ++y)
+			if (field[x][y].what == TARGET)
+				field[x][y].what = EMPTY;
+}
+
+// The target is the set of blocks which the currently falling block
+// will occupy when it lands. It is an aid for beginners.
+void
+BlockOps::generateTarget ()
+{
+	if (!useTarget)
+		return;
+
+	clearTarget ();
+
+	// FIXME: Check that this is actually guaranteed
+	// to terminate (i.e. posx, posy, blocknr and rot 
+	// are guaranteed to be valid).
+	int n = 0;
+	do {
+		n++;
+	} while (blockOkHere (posx, posy + n, blocknr, rot));
+	n--;
+
+	// Mark the relevant places.
+	putBlockInField (posx, posy + n, blocknr, rot, TARGET);
+}
+
+void 
+BlockOps::setUseTarget (bool use)
+{
+	useTarget = use;
+	if (!useTarget)
+		clearTarget ();
+}
+
 int
 BlockOps::dropBlock()
 {
@@ -209,17 +244,8 @@ BlockOps::eliminateLine(int l)
 	{
 		for (int x = 0; x < COLUMNS; ++x)
 		{
-			if (field[x][y].item)
-				gtk_object_destroy(GTK_OBJECT(field[x][y].item));
-
 			field[x][y] = field[x][y - 1];
-			if (field[x][y].item)
-				field[x][y].item = generateItem(x, y, field[x][y].color);
 
-			if (field[x][y - 1].item)
-				gtk_object_destroy(GTK_OBJECT(field[x][y - 1].item));
-
-			field[x][y - 1].item = NULL;
 			field[x][y - 1].what = EMPTY;
 			field[x][y - 1].color = 0;
 		}
@@ -272,14 +298,14 @@ BlockOps::generateFallingBlock()
 	blocknr = blocknr_next == -1 ? g_random_int_range(0, tableSize) :
 		blocknr_next;
 	rot = rot_next == -1 ? g_random_int_range(0, 4) : rot_next;
-	int cn = random_block_colors ? g_random_int_range(0, nr_of_colors) :
-		blocknr % nr_of_colors;
+	int cn = random_block_colors ? g_random_int_range(0, NCOLOURS) :
+		blocknr % NCOLOURS;
 	color = color_next == -1 ? cn : color_next;
 	
 	blocknr_next = g_random_int_range(0, tableSize);
 	rot_next = g_random_int_range(0, 4);
-	color_next = random_block_colors ? g_random_int_range(0, nr_of_colors) :
-		blocknr_next % nr_of_colors;
+	color_next = random_block_colors ? g_random_int_range(0, NCOLOURS) :
+		blocknr_next % NCOLOURS;
 	
 	if (!blockOkHere(posx, posy, blocknr, rot))
 		return false;
@@ -299,28 +325,17 @@ BlockOps::emptyField(int filled_lines, int fill_prob)
 
 		for (int x = 0; x < COLUMNS; ++x)
 		{
-			if (field_initialized && field[x][y].item != 0)
-			{
-				gtk_object_destroy(GTK_OBJECT(field[x][y].item));
-			}
-			field[x][y].item = 0;
 			field[x][y].what = EMPTY;
+			field[x][y].color = 0; 
+
 			if ((y>=(LINES - filled_lines)) && (x != blank) &&
 			    ((g_random_int_range(0, 10)) < fill_prob)) { 
 				field[x][y].what = LAYING;
-				if (nr_of_colors)
-					field[x][y].color = g_random_int_range(0,  nr_of_colors);
-				else
-					// This is in case we're called
-					// before the widgets are set 
-					// up and nr_of_colors is
-					// defined.
-					field[x][y].color = 0; 
-				field[x][y].item = generateItem(x,y,field[x][y].color);
+				field[x][y].color = 
+				  g_random_int_range(0, NCOLOURS);
 			}
 		}
 	}
-	field_initialized = true;
 }
 
 void
@@ -330,34 +345,51 @@ BlockOps::emptyField(void)
 }
 
 void
-BlockOps::putBlockInField(bool erase)
+BlockOps::putBlockInField (int bx, int by, int block, int rotation,
+			   SlotType fill)
 {
 	for (int x = 0; x < 4; ++x)
 	{
 		for (int y = 0; y < 4; ++y)
 		{
-			if (blockTable[blocknr][rot][x][y])
+			if (blockTable[block][rotation][x][y])
 			{
-				field[posx - 2 + x][y + posy].what = erase ? EMPTY : FALLING;
-				if (erase)
-				{
-					GnomeCanvasItem *o = field[posx - 2 + x][y + posy].item;
-					if (o)
-						gtk_object_destroy(GTK_OBJECT(o));
-					field[posx - 2 + x][y + posy].item = 0;
-				}
+				int i = bx - 2 + x;
+				int j = y + by;
+
+				if ((fill == TARGET) && 
+				    (field[i][j].what != EMPTY))
+					return;
+
+				field[i][j].what = fill;
+				if ((fill == FALLING) || (fill == LAYING))
+					field[i][j].color = color;
 				else
-				{
-					field[posx - 2 + x][y + posy].color = color;
-					field[posx - 2 + x][y + posy].item = 
-						generateItem(posx - 2 + x, y + posy, 
-									 field[posx - 2 + x][y + posy].color);
-				}
+					field[i][j].color = 0;
 			}
 		}
 	}
 }
 
+// This is now just a wrapper. I'm not sure which version should be
+// used in general: having the field keep track of the block worries
+// me, but I can't say it is definitely wrong.
+void
+BlockOps::putBlockInField (bool erase)
+{
+	if (erase)
+		putBlockInField (posx, posy, blocknr, rot, EMPTY);
+	else
+		putBlockInField (posx, posy, blocknr, rot, FALLING);
+}
 
+bool
+BlockOps::isFieldEmpty (void)
+{
+  for (int x = 0; x < COLUMNS; x++) {
+    if (field[x][LINES-1].what != EMPTY)
+      return false;
+  }
 
-
+  return true;
+}
